///*|----------------------------------------------------------------------------------------------------
// *|            This source code is provided under the Apache 2.0 license      	--
// *|  and is provided AS IS with no warranty or guarantee of fit for purpose.  --
// *|                See the project's LICENSE.md for details.                  					--
// *|           Copyright Refinitiv 2023. All rights reserved.            		--
///*|----------------------------------------------------------------------------------------------------

package com.refinitiv.ema.tutorials.consumer.tutorial6.rcc;

import com.refinitiv.ema.access.*;
import com.refinitiv.ema.rdm.EmaRdm;

/**
 *  This is the main class for RCC RSSL connection.
 *  It implements OmmConsumerClient to receive RSSL Login and Directory connection events via callbacks.
 */
class AppClient implements OmmConsumerClient {
    private TunnelStreamClient _tunnelClient = null;

    /**
     * This function set the TunnelStreamClient from the main Contributor class
     * @param tunnelClient
     */
    public void setTunnelStreamClient(TunnelStreamClient tunnelClient) {
        this._tunnelClient = tunnelClient;
    }

    /**
     * The onRefreshMsg callback method handle the Login and Directory Refresh response messages for the RCC RSSL connection.
     * @param refreshMsg
     * @param event
     */
    @Override
    public void onRefreshMsg(RefreshMsg refreshMsg, OmmConsumerEvent event) {


        try {
            System.out.println("----- Refresh message ----");
            System.out.println(refreshMsg);

            if (_tunnelClient == null) {
                System.out.println("No Tunnel Client");
                return;
            }

            //Process RCC Tunneling connection logic based on incoming RSSL connection domain
            if (refreshMsg.domainType() == EmaRdm.MMT_LOGIN) {
                _tunnelClient.processLoginRefresh(refreshMsg, event);
            } else if (refreshMsg.domainType() == EmaRdm.MMT_DIRECTORY) {
                _tunnelClient.processSourceDirectoryRefresh(refreshMsg, event);
            }

        } catch (OmmException excp) {
            System.out.println(excp.getMessage());
        }
    }

    /**
     * The onUpdateMsg callback method handle the Login and Directory Update response messages for the RCC RSSL connection.
     * @param updateMsg
     * @param event
     */
    @Override
    public void onUpdateMsg(UpdateMsg updateMsg, OmmConsumerEvent event) {

        try {
            System.out.println("----- Update message ----");
            System.out.println(updateMsg);

            if (_tunnelClient == null) {
                System.out.println("No Tunnel Client");
                return;
            }

            //Process RCC Tunneling connection logic based on incoming RSSL connection domain
            if (updateMsg.domainType() == EmaRdm.MMT_DIRECTORY) {
                _tunnelClient.processSourceDirectoryUpdate(updateMsg, event);
            } else {
                System.out.println("Unhandled domain: " + updateMsg.domainType());
            }

        } catch (OmmException excp) {
            System.out.println(excp.getMessage());
        }
    }

    /**
     * The onStatusMsg callback method handle the Login and Directory Status response messages for the RCC RSSL connection.
     * @param  statusMsg
     * @param event
     */
    @Override
    public void onStatusMsg(StatusMsg statusMsg, OmmConsumerEvent event) {

        try {
            System.out.println("----- Status message ----");
            System.out.println(statusMsg);

            if (_tunnelClient == null) {
                System.out.println("No Tunnel Client");
                return;
            }

            //Process RCC Tunneling connection logic based on incoming RSSL connection domain
            if (statusMsg.domainType() == EmaRdm.MMT_LOGIN) {
                _tunnelClient.processLoginStatus(statusMsg, event);
            } else if (statusMsg.domainType() == EmaRdm.MMT_DIRECTORY) {
                _tunnelClient.processSourceDirectoryStatus(statusMsg, event);
            } else {
                System.out.println("Unhandled domain: " + statusMsg.domainType());
            }
        } catch (OmmException excp) {
            System.out.println(excp.getMessage());
        }

    }

    @Override
    public void onAckMsg(AckMsg ackMsg, OmmConsumerEvent consumerEvent) {
        System.out.println("----- Ack message ----");
        System.out.println(ackMsg);
    }

    @Override
    public void onGenericMsg(GenericMsg genericMsg, OmmConsumerEvent consumerEvent) {
        System.out.println("----- Generic message ----");
    }

    @Override
    public void onAllMsg(Msg msg, OmmConsumerEvent consumerEvent) {
    }

}
