///*|----------------------------------------------------------------------------------------------------
// *|            This source code is provided under the Apache 2.0 license      	--
// *|  and is provided AS IS with no warranty or guarantee of fit for purpose.  --
// *|                See the project's LICENSE.md for details.                  					--
// *|           Copyright Refinitiv 2023. All rights reserved.            		--
///*|----------------------------------------------------------------------------------------------------

package com.refinitiv.ema.tutorials.consumer.tutorial6.rcc;

import com.refinitiv.ema.access.*;
import com.refinitiv.ema.rdm.EmaRdm;

/**
 *  This is the main class for RCC RSSL-Tunnel Stream connection.
 *  It implements OmmConsumerClient to receive Tunnel Stream connection events via callbacks.
 *  It starts and login a Tunnel Stream once it gets the Directory Refresh Response message from RCC to guarantee that a Service IS Up.
 *  It starts posting a Market Price data to RCC once it gets a Tunnel Stream Login Refresh and ACK  Response messages from RCC.
 */
public class TunnelStreamClient implements OmmConsumerClient {

    private OmmConsumer _ommConsumer;
    private long _tunnelStreamHandle, _subStreamHandle;
    private int _postStreamID;
    private String trceUser = "";
    private String trcePass = "";
    private String trceItem = "";
    private String trceServiceName = "";
    private boolean _subItemOpen;
    int _postID = 0;
    int _bid = 0, _ask = 0;

    TunnelStreamClient(String trceUser, String trcePass, String trceServiceName, String trceItem) {
        this.trceUser = trceUser;
        this.trcePass = trcePass;
        this.trceServiceName = trceServiceName;
        this.trceItem = trceItem;
        this._subItemOpen = false;

        _postID = 1;
        _bid = 340;
        _ask = 341;
    }

    /**
     * This method used for setting OmmConsumer from Contributor.java
     * @param ommConsumer OMMConsumer Object
     */
    public void setOmmConsumer(OmmConsumer ommConsumer) {
        _ommConsumer = ommConsumer;
    }


    @Override
    public void onRefreshMsg(RefreshMsg refreshMsg, OmmConsumerEvent event) {
        System.out.println("----- TunnelStreamClient Refresh message ----");
        System.out.println(refreshMsg);

        //If this is a Tunnel Stream Login Refresh Response message and the Stream is UP, sends a Post message
        if (refreshMsg.domainType() == EmaRdm.MMT_LOGIN && refreshMsg.state().streamState() == OmmState.StreamState.OPEN && refreshMsg.state().dataState() == OmmState.DataState.OK) {
            // 3. Login accepted, app can post data now
            System.out.println("Login accepted, starting posting...");

            _postStreamID = refreshMsg.streamId();
            postMessage();
        } else {
            System.out.println("Stream not open");
        }
    }

    @Override
    public void onUpdateMsg(UpdateMsg updateMsg, OmmConsumerEvent event) {
        System.out.println("----- TunnelStreamClient Update message ----");
    }

    @Override
    public void onStatusMsg(StatusMsg statusMsg, OmmConsumerEvent event) {
        System.out.println("----- TunnelStreamClient Status message ----");
        System.out.println(statusMsg);

        //If this is a Tunnel Stream Status message and the Stream is UP, sends a OMM Login request message over this Tunnel Stream
        if (event.handle() == _tunnelStreamHandle && statusMsg.hasState() && statusMsg.state().streamState() == OmmState.StreamState.OPEN) {
            System.out.println("************* Login to RCC ******************** " + statusMsg.streamId());
            loginTunnelStream(statusMsg.streamId(), statusMsg.serviceId());
        }
    }

    @Override
    public void onGenericMsg(GenericMsg genericMsg, OmmConsumerEvent event) {
        System.out.println("----- Generic message ----");
    }

    @Override
    public void onAckMsg(AckMsg ackMsg, OmmConsumerEvent event) {
        System.out.println("----- Ack message ----");
        System.out.println(ackMsg);

        System.out.println("Continue posting...");
        //Receive ACK (Post success), continue Posting
        postMessage();
    }

    @Override
    public void onAllMsg(Msg msg, OmmConsumerEvent event) {

    }

    /**
     *  Creates the OMM Post message for RCC and sends it over a Tunnel Stream
     */
    private void postMessage() {
        try {
            Thread.sleep(300);
        } catch (Exception e) {
        }

        // populate the contributed FIDs and values
        FieldList nestedFieldList = EmaFactory.createFieldList();
        nestedFieldList.add(EmaFactory.createFieldEntry().real(22, _bid++, OmmReal.MagnitudeType.EXPONENT_NEG_1));
        nestedFieldList.add(EmaFactory.createFieldEntry().real(25, _ask++, OmmReal.MagnitudeType.EXPONENT_NEG_1));

        // create an update message for our item
        UpdateMsg nestedUpdateMsg = EmaFactory.createUpdateMsg()
                .streamId(_postStreamID)
                .name(trceItem)
                .payload(nestedFieldList);

        // post this market price message
        _ommConsumer.submit(EmaFactory.createPostMsg()
                .streamId(_postStreamID)
                .postId(_postID++)
                .domainType(EmaRdm.MMT_MARKET_PRICE)
                .solicitAck(true)
                .complete(true)
                .payload(nestedUpdateMsg), _subStreamHandle);
    }

    /**
     * Create the OMM Login request message and sends it to RCC over Tunnel Stream.
     * The message requires RCC Username (trceUser) and Password (trcePass).
     * @param streamId
     * @param serviceId
     */
    private void loginTunnelStream(int streamId, int serviceId) {

        System.out.println("TunnelStreamClient Login Tunnel Stream");

        // Create the OM Login Request Message
        ElementList elementList = EmaFactory.createElementList();
        elementList.add(EmaFactory.createElementEntry().ascii("Password", this.trcePass));

        ReqMsg rMsg = EmaFactory.createReqMsg()
                .domainType(EmaRdm.MMT_LOGIN)
                .name(this.trceUser)
                .attrib(elementList)
                // .privateStream(true)
                .serviceId(serviceId)
                .streamId(streamId);

        System.out.println("Sending client login request...");
        // get events from login substream
        _subStreamHandle = _ommConsumer.registerClient(rMsg, this, 1, _tunnelStreamHandle);
        _subItemOpen = true;
    }

    /**
     * Initiate a Tunnel Stream over the current RSSL connection to RCC.
     * The message requires RCC Service Name (trceServiceName).
     * The function is called when this class confirms the RCC service is UP from the Directory Refresh message.
     */
    private void startTunnelStream() {
        System.out.println("TunnelStreamClient Start Tunnel Stream");
        try {
            ClassOfService cos = EmaFactory.createClassOfService()
                    .authentication(EmaFactory.createCosAuthentication().type(CosAuthentication.CosAuthenticationType.NOT_REQUIRED))
                    .dataIntegrity(EmaFactory.createCosDataIntegrity().type(CosDataIntegrity.CosDataIntegrityType.RELIABLE))
                    .flowControl(EmaFactory.createCosFlowControl().type(CosFlowControl.CosFlowControlType.BIDIRECTIONAL).recvWindowSize(1200))
                    .guarantee(EmaFactory.createCosGuarantee().type(CosGuarantee.CosGuaranteeType.NONE));

            TunnelStreamRequest tsr = EmaFactory.createTunnelStreamRequest()
                    .classOfService(cos)
                    .domainType(EmaRdm.MMT_SYSTEM)
                    .name("TUNNEL1")
                    .serviceName(this.trceServiceName);

            _tunnelStreamHandle = _ommConsumer.registerClient(tsr, this);
        } catch (Exception excp) {
            System.out.println(excp.getMessage());
        }
    }

    /**
     * Process RCC RSSL connection Login domain Refresh message.
     * @param refreshMsg
     * @param event
     */
    public void processLoginRefresh(RefreshMsg refreshMsg, OmmConsumerEvent event) {
        System.out.println("TunnelStreamClient processLoginStatus");

        if (refreshMsg.state().streamState() == OmmState.StreamState.OPEN && refreshMsg.state().dataState() == OmmState.DataState.OK) {
            System.out.println("Tunnel Stream Login accepted");
            _tunnelStreamHandle = 0;
            _subStreamHandle = 0;
        }
    }

    /**
     * Process RCC RSSL connection Login domain Status message.
     * @param statusMsg
     * @param event
     */
    public void processLoginStatus(StatusMsg statusMsg, OmmConsumerEvent event) {
        System.out.println("TunnelStreamClient processLoginStatus");

        if (statusMsg.state().streamState() == OmmState.StreamState.OPEN && statusMsg.state().dataState() == OmmState.DataState.SUSPECT) {
            _tunnelStreamHandle = 0;
            _subStreamHandle = 0;
        }
    }

    /**
     * Process RCC RSSL connection Directory domain Refresh message.
     * @param refreshMsg
     * @param event
     */
    public void processSourceDirectoryRefresh(RefreshMsg refreshMsg, OmmConsumerEvent event) {
        System.out.println("TunnelStreamClient processSourceDirectoryRefresh");

        if (refreshMsg.state().streamState() == OmmState.StreamState.OPEN && refreshMsg.state().dataState() == OmmState.DataState.OK && _tunnelStreamHandle == 0) {
            System.out.println("RSSL encrypted connection Login accepted and Directory is UP, starting Tunnel Stream...");
            startTunnelStream();
        }
    }

    /**
     * Process RCC RSSL connection Directory domain Update message.
     * @param updateMsg
     * @param event
     */
    public void processSourceDirectoryUpdate(UpdateMsg updateMsg, OmmConsumerEvent event) {
        System.out.println("TunnelStreamClient processSourceDirectoryUpdate");
        if (_tunnelStreamHandle == 0) {
            startTunnelStream();
        }
    }

    /**
     * Process RCC RSSL connection Directory domain Status message.
     * @param statusMsg
     * @param event
     */
    public void processSourceDirectoryStatus(StatusMsg statusMsg, OmmConsumerEvent event) {
        System.out.println("TunnelStreamClient processSourceDirectoryStatus");
    }
}
